"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.enrichDuplicateTitles = exports.optimizeTocArray = void 0;
/**
 * Utility function to efficiently handle large table of contents arrays
 * by removing duplicates and optionally limiting size while preserving hierarchy
 */
const optimizeTocArray = (tocEntries, reporter, options = {}) => {
    const { maxSize = 2000, // Maximum entries to include
    warnThreshold = 300, // Threshold to issue warning
    removeDuplicates = true, // Whether to remove duplicates
    enhanceDuplicates = true, // Whether to enhance duplicate titles with context
    buildHierarchy = true, // Whether to build the hierarchy tree
     } = options;
    if (!tocEntries || tocEntries.length === 0) {
        return [];
    }
    // Track memory usage for large TOCs
    if (tocEntries.length > warnThreshold) {
        reporter.warn(`Large table of contents detected (${tocEntries.length} items). This might affect performance.`);
    }
    // Pre-process all entries to ensure proper levels
    const enrichedTocEntries = tocEntries.map((entry) => {
        if (!entry.level) {
            const level = entry.type === "heading_1"
                ? 1
                : entry.type === "heading_2"
                    ? 2
                    : entry.type === "heading_3"
                        ? 3
                        : 0;
            return { ...entry, level };
        }
        return entry;
    });
    // Remove duplicates if requested
    let processedToc = enrichedTocEntries;
    if (removeDuplicates) {
        const startTime = Date.now();
        const uniqueMap = new Map();
        // Use the hash to ensure uniqueness, but preserve context
        for (const entry of enrichedTocEntries) {
            uniqueMap.set(entry.hash, entry);
        }
        processedToc = Array.from(uniqueMap.values());
        const removedCount = enrichedTocEntries.length - processedToc.length;
        const processTime = Date.now() - startTime;
        if (removedCount > 0) {
            reporter.info(`Removed ${removedCount} duplicate TOC entries in ${processTime}ms.`);
        }
    }
    // Add context to duplicate title entries for better display
    if (enhanceDuplicates) {
        (0, exports.enrichDuplicateTitles)(processedToc, reporter);
    }
    // Limit size if necessary
    if (processedToc.length > maxSize) {
        reporter.warn(`Table of contents exceeds maximum size (${processedToc.length} > ${maxSize}). Truncating.`);
        processedToc = processedToc.slice(0, maxSize);
    }
    // Build the hierarchical structure if requested
    if (buildHierarchy) {
        // First level sort - by level, then by approximate order
        processedToc.sort((a, b) => {
            if ((a.level || 0) !== (b.level || 0)) {
                return (a.level || 0) - (b.level || 0);
            }
            return 0; // Keep original order for same level
        });
        // 확인을 위한 로깅
        const h1Count = processedToc.filter((item) => item.level === 1).length;
        const h2Count = processedToc.filter((item) => item.level === 2).length;
        const h3Count = processedToc.filter((item) => item.level === 3).length;
        reporter.info(`TOC structure: ${h1Count} h1, ${h2Count} h2, ${h3Count} h3 headings`);
    }
    return processedToc;
};
exports.optimizeTocArray = optimizeTocArray;
/**
 * Enhances TOC entries with contextual titles to disambiguate duplicate titles
 */
const enrichDuplicateTitles = (tocEntries, reporter) => {
    if (!tocEntries || tocEntries.length === 0)
        return;
    // Find duplicate titles
    const titleCounts = new Map();
    const duplicateTitles = new Set();
    // Count occurrences of each title
    for (const entry of tocEntries) {
        const count = (titleCounts.get(entry.title) || 0) + 1;
        titleCounts.set(entry.title, count);
        if (count > 1) {
            duplicateTitles.add(entry.title);
        }
    }
    if (duplicateTitles.size === 0)
        return;
    reporter.info(`Found ${duplicateTitles.size} duplicate title(s) in TOC. Adding context...`);
    // Build parent map for fast lookups
    const entryMap = new Map();
    for (const entry of tocEntries) {
        entryMap.set(entry.hash, entry);
    }
    // Process each entry to ensure all duplicate titles have context
    for (const entry of tocEntries) {
        if (duplicateTitles.has(entry.title) && !entry.contextTitle) {
            let context = "";
            let parentEntry;
            // Find parent context
            if (entry.parentHash && entryMap.has(entry.parentHash)) {
                parentEntry = entryMap.get(entry.parentHash);
                context = parentEntry?.title || "";
            }
            // Add context to title
            if (context) {
                entry.contextTitle = `${entry.title} (${context})`;
            }
            else {
                entry.contextTitle = entry.title;
            }
        }
    }
};
exports.enrichDuplicateTitles = enrichDuplicateTitles;
